@@ -41,21 +41,22 @@
 #' str(inp)
 #' }
 addKnownMargins <- function(inp, margins) {
+  N <- NULL
   dataP <- inp@pop@data
   margins <- as.data.frame(margins)
-  if ( !class(margins) == "data.frame" ) {
+  if ( !inherits(margins, "data.frame" ) ) {
     stop("input argument 'margins' must inherit class 'data.frame'!\n")
   }
   if ( any(duplicated(margins)) ) {
     stop("'margins' must not contain duplicated rows!\n")
   }
-  if ( !class(margins[,ncol(margins)])[1] %in% c("integer","numeric") ) {
+  if ( !(inherits(margins[,ncol(margins)], "integer") | inherits(margins[,ncol(margins)],"numeric") ) ) {
     stop("last column of input 'margins' must contain the numbers (must be numeric)!\n")
   } else {
     vals <- margins[,ncol(margins)]
     margins <- margins[,-ncol(margins), drop=FALSE]
   }
-  if ( !class(inp) == "simPopObj" ) {
+  if ( !inherits(inp, "simPopObj" ) ){
     stop("input argument 'inp' must be of class 'simPopObj'!\n")
   }
   if ( !all(colnames(margins) %in% colnames(dataP)) ) {
@@ -88,7 +89,8 @@ addKnownMargins <- function(inp, margins) {
   if ( length(ind) > 0 ) {
     frame$N[ind] <- 0
   }
-
+  frame <- frame[N>0]
+  
   if ( !is.null(inp@table) ) {
     message("Note: currently stored marginals/totals are going to be overwritten!\n")
   }
