@@ -17,7 +17,7 @@ generateValues <- function(dataSample, dataPop, params) {
     invisible(unlist(head(dataSample[,cur.var,with=FALSE],1)))
   }else{
     # temporarily recode response vector
-    dataSample[,(cur.var):=cleanFactor(.SD),.SDcols=cur.var]
+    dataSample[,c(cur.var):=lapply(.SD,cleanFactor),.SDcols=c(cur.var)]
     levelsResponse <- levels(unlist(dataSample[,cur.var,with=FALSE]))
 
     #indices for unique occurence
@@ -71,7 +71,11 @@ generateValues <- function(dataSample, dataPop, params) {
     }else if ( meth %in% c("ranger") ) {
       probs <- predict(mod,data=newdata,type="response")$predictions
       colnames(probs) <- mod$forest$levels
-	  }
+    }else if ( meth %in% c("xgboost") ) {
+      probs <- predict(mod, newdata=xgb.DMatrix(data = model.matrix(~.+0,data = setDT(newdata))))
+      # create matrix from prediction array
+      probs <- matrix(probs, nrow = nrow(newdata), ncol = mod[["params"]][["num_class"]], byrow = T)
+    }
     #if ( meth %in% "naivebayes" ) {
     #  probs <- predict(mod, newdata=newdata, type="raw")
     #}
@@ -106,6 +110,10 @@ generateValues <- function(dataSample, dataPop, params) {
     }
     # generate realizations for each combination
 
+    if(meth == c("xgboost")) {
+      ind <- 1:length(levelsResponse)
+    }
+    
     if ( length(exclude) == 0 ) {
       ncomb <- as.integer(sapply(indGrid, length))
       sim <- lapply(1:length(ncomb), resample, ncomb, probs)
@@ -133,12 +141,19 @@ generateValues_distribution <- function(dataSample, dataPop, params) {
 
   # population data
   splitS <- split(1:nrow(dataSample), dataSample[, basic, with=FALSE], drop=TRUE)
+
   pSplit <- lapply(splitS, function(i) {
-        tmp <- tableWt(dataSample[i, additional, with=FALSE], dataSample[[w]][i])
-        tmp <- as.data.frame(tmp)
-        p <- ncol(tmp)
-        tmp[, p]/sum(tmp[, p])
-      })
+    if(!is.null(w)){
+      wvec <- dataSample[[w]][i] 
+    }else{
+      wvec <- NULL
+    }
+    tmp <- tableWt(dataSample[i, additional, with=FALSE], wvec)
+    tmp <- as.data.frame(tmp)
+    p <- ncol(tmp)
+    tmp[, p]/sum(tmp[, p])
+  })
+
   splitP <- split(1:nrow(dataPop), dataPop[, basic, with=FALSE])
   NSplit <- sapply(splitP, length)
   # in sample, observations with NAs have been removed to fit the
@@ -146,7 +161,10 @@ generateValues_distribution <- function(dataSample, dataPop, params) {
   whichP <- which(names(splitP) %in% names(splitS))
   # generate realizations for each combination
   sim <- as.list(rep.int(NA, length(splitP)))
-  sim[whichP] <- mapply(spSample, NSplit[whichP], pSplit, SIMPLIFY=FALSE)
+  for(i in seq_along(whichP)){
+    sim[[whichP[i]]] <- spSample(NSplit[whichP][i],pSplit[[i]])
+  }
+  #sim[whichP] <- mapply(spSample, NSplit[whichP], pSplit, SIMPLIFY=FALSE)
   sim <- unsplit(sim, dataPop[, basic, with=FALSE])
   sim <- grid[sim,,drop=FALSE]
   rownames(sim) <- rownames(dataPop)
@@ -184,6 +202,7 @@ generateValues_distribution <- function(dataSample, dataPop, params) {
 #' \code{"ctree"}  for using Classification trees
 #' \code{"cforest"}  for using random forest (implementation in package party)
 #' \code{"ranger"}  for using random forest (implementation in package ranger)
+#' \code{"xgboost"}  for using xgboost (implementation in package xgboost)
 #' @param limit if \code{method} is \code{"multinom"}, this can be used to
 #' account for structural zeros. If only one additional variable is requested,
 #' a named list of lists should be supplied. The names of the list components
@@ -233,12 +252,13 @@ generateValues_distribution <- function(dataSample, dataPop, params) {
 #' number generator to be restored.
 #' @param verbose set to TRUE if additional print output should be shown.
 #' @param by defining which variable to use as split up variable of the estimation. Defaults to the strata variable.
+#' @param model_params NULL or a named list which can contain model specific parameters which will be passed onto the function call for the respective model. 
 #' @return An object of class \code{\linkS4class{simPopObj}} containing survey
 #' data as well as the simulated population data including the categorical
 #' variables specified by argument \code{additional}.
 #' @note The basic household structure needs to be simulated beforehand with
 #' the function \code{\link{simStructure}}.
-#' @author Bernhard Meindl, Andreas Alfons, Stefan Kraft, Alexander Kowarik, Matthias Templ
+#' @author Bernhard Meindl, Andreas Alfons, Stefan Kraft, Alexander Kowarik, Matthias Templ, Siro Fritzmann
 #' @references 
 #' B. Meindl, M. Templ, A. Kowarik, O. Dupriez (2017) Simulation of Synthetic Populations for Survey Data Considering Auxiliary
 #' Information. \emph{Journal of Statistical Survey}, \strong{79} (10), 1--38. \doi{10.18637/jss.v079.i10}
@@ -260,10 +280,10 @@ generateValues_distribution <- function(dataSample, dataPop, params) {
 #' simPop
 #' }
 simCategorical <- function(simPopObj, additional,
-    method=c("multinom", "distribution","ctree","cforest","ranger"),
-    limit=NULL, censor=NULL, maxit=500, MaxNWts=1500,
-    eps=NULL, nr_cpus=NULL, regModel=NULL, seed=1,
-    verbose=FALSE,by="strata") {
+    method = c("multinom", "distribution","ctree","cforest","ranger","xgboost"),
+    limit = NULL, censor = NULL, maxit = 500, MaxNWts = 1500,
+    eps = NULL, nr_cpus = NULL, regModel = NULL, seed = 1,
+    verbose = FALSE,by = "strata",model_params = NULL) {
 
   x <- newAdditionalVarible <- NULL
 
@@ -286,8 +306,8 @@ simCategorical <- function(simPopObj, additional,
     stop("variables already exist in the population!\n")
   }
 
-  if ( (length(regModel)==1|class(regModel)=="formula") & length(additional)>1 ) {
-    if(class(regModel)=="formula"){
+  if ( (length(regModel)==1|inherits(regModel, "formula")) & length(additional)>1 ) {
+    if(inherits(regModel, "formula")){
       regModelL <- list()
       for(i in seq_along(additional)){
         regModelL[[i]] <- regModel
@@ -298,7 +318,7 @@ simCategorical <- function(simPopObj, additional,
     }
   }
   if (!is.null(regModel) ) {
-    if ( class(regModel)=="formula" ) {
+    if ( inherits(regModel, "formula") ) {
       regModel <- list(regModel)
     }
   }
@@ -315,6 +335,27 @@ simCategorical <- function(simPopObj, additional,
     if ( is.null(regModel) ) {
       regModel <- rep("basic", length(additional))
     }
+    param.cmd <- NULL
+    if(!is.null(model_params)){
+      if(!(is.list(model_params) & !is.null(names(model_params)))){
+        stop("Parameter model_params must be a named list where each entry refers to a parameter of the model chosen by `method`")
+      }
+     
+      all_model_params <- as.list(args(get(method)))
+      
+      names_params <- names(model_params)
+      names_all <- names(all_model_params)
+      if(any(!names_params %in% names_all) & !"..."%in%names_all){
+        stop("The following are not parameters of method ",method,"\n ",paste(names_params[!names_params %in% names_all],collapse=", "))
+      }
+      
+      # build parameter call for model
+      param.cmd <- paste(names_params,paste0("params$model_extra$",names_params),sep="=",collapse=", ")
+    }
+    
+    
+    
+    
   }
   # parameters for parallel computing
   if(by=="strata"){
@@ -451,6 +492,10 @@ simCategorical <- function(simPopObj, additional,
       if(!dataS@ispopulation){
         formula.cmd <- paste0(formula.cmd,", weights=", dataS@weight)
       }
+      if(!is.null(param.cmd)){
+        formula.cmd <- paste0(formula.cmd,", ",param.cmd)
+      }
+      
       formula.cmd <- paste0(formula.cmd,", data=dataSample, trace=FALSE",
                             ", maxit=",maxit, ", MaxNWts=", MaxNWts,"))")
       
@@ -463,6 +508,9 @@ simCategorical <- function(simPopObj, additional,
       if(!dataS@ispopulation){
         formula.cmd <- paste0(formula.cmd,", weights=as.integer(dataSample$", dataS@weight,")")
       }
+      if(!is.null(param.cmd)){
+        formula.cmd <- paste0(formula.cmd,", ",param.cmd)
+      }
       formula.cmd <- paste0(formula.cmd,
                            ", data=dataSample))")
       if(verbose) cat("we are running recursive partitioning:\n")
@@ -472,7 +520,10 @@ simCategorical <- function(simPopObj, additional,
       formula.cmd <- paste(i, "~", paste(predNames, collapse = " + "))
       formula.cmd <- paste("suppressWarnings(cforest(", formula.cmd)
       if(!dataS@ispopulation){
-        formula.cmd <- paste0(formula.cmd,", weights=as.integer(dataSample$", dataS@weight,")")
+        formula.cmd <- paste0(formula.cmd,", weights=as.numeric(dataSample$", dataS@weight,")")
+      }
+      if(!is.null(param.cmd)){
+        formula.cmd <- paste0(formula.cmd,", ",param.cmd)
       }
       formula.cmd <- paste0(formula.cmd,", data=dataSample))")
       if(verbose) cat("we are running random forest classification (cforest):\n")
@@ -483,11 +534,88 @@ simCategorical <- function(simPopObj, additional,
 		  formula.cmd <- paste("suppressWarnings(ranger(", formula.cmd)
 		  if(!dataS@ispopulation){
   		  formula.cmd <- paste0(formula.cmd,", case.weights=dataSample$", dataS@weight)
-	  	}
+		  }
+		  if(!is.null(param.cmd)){
+		    formula.cmd <- paste0(formula.cmd,", ",param.cmd)
+		  }
 		  formula.cmd <- paste0(formula.cmd, ", data=dataSample,probability=TRUE))", sep="")
 	  	if(verbose) cat("we are running random forest (ranger):\n")
 		  if(verbose) cat(strwrap(cat(gsub("))",")",gsub("suppressWarnings[(]","",formula.cmd)),"\n"), 76), sep = "\n")
-	}
+    }else if ( method == "xgboost" ) {
+      
+      # simulation via xgboost
+      if(verbose) cat("we are running xgboost:\n")
+      
+      # set xgb verbose level
+      if(verbose){
+        xgb_verbose <- 1
+      }else{
+        xgb_verbose <- 0
+      }
+      
+      if(!dataS@ispopulation){
+        weight_str <- paste0("as.numeric(dataSample$", dataS@weight, ")")
+        # xgb_weight <- paste0(", info = list(\"weight\" = (",weight_str," - min(", weight_str, "))
+        #                      / (max(", weight_str, ") - min(", weight_str, ")))")
+        # xgb_weight <- paste0(", info = list(\"weight\" = (",weight_str," / max(", weight_str, ")))")
+        xgb_weight <- paste0(", info = list(\"weight\" = (",weight_str,"))")
+      }else{
+        xgb_weight <- ""
+      }
+      
+      pred_names <- paste(predNames, collapse = "\",\"")
+      train <- paste0("xgb.DMatrix(data = model.matrix(~.+0,data = setDT(dataSample)[,c(\"", pred_names,"\"), with=F]),
+                                   label = as.numeric(dataSample$",i,") - 1
+                                        ", xgb_weight,")")
+      
+      # Default values
+      nrounds <- 100
+      early_stopping_rounds <- 10
+      xgb_hyper_params <- "list(nthread = 6,
+                                eta = 0.3,
+                                max_depth = 32,
+                                min_child_weight = 1,
+                                gamma = 0,
+                                subsample = 1,
+                                objective = \"multi:softprob\",
+                                eval_metric = \"mlogloss\")"
+      
+      if(!is.null(model_params)){
+        
+        xgb_hyper_params <- eval(parse(text=xgb_hyper_params))
+        for(hyper in names(xgb_hyper_params)){
+          if(is.null(model_params[[hyper]])){
+            model_params[[hyper]] <- xgb_hyper_params[[hyper]]
+          }
+        }
+        xgb_hyper_params <- "params$model_extra"
+        
+        if(!is.null(model_params$nrounds)){
+          nrounds <- model_params$nrounds
+          model_params$nrounds <- NULL
+        }
+        
+        if(!is.null(model_params$early_stopping_rounds)){
+          early_stopping_rounds <- model_params$early_stopping_rounds
+          model_params$early_stopping_rounds <- NULL
+        }
+      }
+      
+      xgb_params <- paste0("nrounds = ", nrounds,",
+                            watchlist = list(train = ", train, ",
+                                             test = ", train, "), 
+                            early_stopping_rounds = ", early_stopping_rounds,",
+                            print_every_n = 10,")
+      
+      command <- paste0("xgb.train(",train, ", ",
+                                    xgb_params,
+                                    "num_class = ", length(levelsResponse), ", ",
+                                    "verbose = ", xgb_verbose, ", ",
+                                    "params = ", xgb_hyper_params, ")")
+      
+      formula.cmd <- command
+      if(verbose) cat(strwrap(cat(gsub("))",")",gsub("suppressWarnings[(]","",formula.cmd)),"\n"), 76), sep = "\n")
+    }
     #if ( method == "naivebayes" ) {
     #  formula.cmd <- paste(i, "~", paste(predNames, collapse = " + "))
     #  formula.cmd <- paste("naiveBayes(", formula.cmd, ", data=dataSample, usekernel=TRUE)", sep="")
@@ -516,7 +644,7 @@ simCategorical <- function(simPopObj, additional,
     params$limit <- limit
     params$censor <- censor
     params$levelsResponse <- levelsResponse
-
+    params$model_extra <- model_params
     # windows
     if ( parallel ) {
       if ( have_win ) {
