@@ -379,14 +379,75 @@ generateValues_binary <- function(dataSample, dataPop, params) {
   unsplit(sim, dataPop, drop=TRUE)
 }
 
+generateValues_xgboost <- function(dataSample, dataPop, params) {
+  
+  res <- NULL
+  
+  predNames <- params$predNames
+  name <- params$name
+  typ <- params$typ
+  formula <- params$formula
+  weight <- params$weight
+  command <- params$command
+  residual <- params$residuals
+  strata <- params$strata
+  
+  
+  # remove strata from prediction, because xgboost can not handle factors with only one level
+  predNames <- predNames[predNames != strata]
+  
+  # Check if a factor has only one level
+  check_levels <- sapply (predNames, function(nam) {
+      length(levels(dataSample[[nam]])) == 1
+  })
+  
+  if(any(check_levels)){
+    stop(paste0(predNames[which(check_levels)]),
+         " has only one level, XGBoost can't work with categorical variables with one level")
+  }
+  
+  mod <- eval(parse(text=command))
+
+  # set sample factor levels to population
+  ind <- match(colnames(dataPop), colnames(dataSample))
+  for ( i in 1:length(ind) ) {
+    if (is.factor(unlist(dataPop[,i,with=FALSE]))) {
+      dataPop[,colnames(dataPop)[i]:=factor(as.character(unlist(dataPop[,colnames(dataPop)[i],with=FALSE])),levels(dataSample[[ind[i]]]))]
+    }
+  }
+
+  new_data <- xgb.DMatrix(data = model.matrix(~.+0, data = model.frame(dataPop[, predNames, with = FALSE],  na.action=na.pass)), missing = NA)
+  pred <- predict(mod,
+                  newdata=new_data)
+  
+  # if residuals is true, calculate in-sample residuals and add an error term to the predictions
+  if(residual){
+    predSample <- predict(mod,
+                          newdata=xgb.DMatrix(data = model.matrix(~.+0, data = model.frame(dataSample[, predNames, with = FALSE],  na.action=na.pass)),
+                                              missing = NA,
+                                              info = list(weight = as.numeric(dataSample[, weight, with = FALSE]))))
+    resSample <- cbind(dataSample, predSample)
+    
+    resSample[, res := predSample - .SD, .SDcols = name]
+    target <- resSample[, name, with = FALSE][[1]]
+    
+    error <- sample(resSample$res, nrow(dataPop), replace = TRUE)
+    pred <- pred + error
+  }
+  
+  return(pred)
+}
+
 genVals <- function(dataSample, dataPop, params, typ, response) {
+  
   # unify level-set of predictors
   for ( i in params$predNames ) {
     dataSample[[i]] <- cleanFactor(dataSample[[i]])
     dataPop[[i]] <- cleanFactor(dataPop[[i]])
   }
 
-  if ( !typ %in% c("multinom","lm","binary","poisson") ) {
+
+  if ( !typ %in% c("multinom","lm","binary","poisson","xgboost") ) {
     stop("unsupported value for argument 'type' in genVals()\n")
   }
   # Check wheter all response values are the same
@@ -402,6 +463,8 @@ genVals <- function(dataSample, dataPop, params, typ, response) {
     res <- generateValues_multinom(dataSample, dataPop, params)
   }else if ( typ=="poisson") {
     res <- generateValues_poisson(dataSample, dataPop, params)
+  }else if ( typ=="xgboost") {
+    res <- generateValues_xgboost(dataSample, dataPop, params)
   }
   res
 }
@@ -413,62 +476,86 @@ runModel <- function(dataS, dataP, params, typ) {
   indStrata <- params$indStrata
   predNames <- params$predNames
   additional <- unique(c(params$additional, params$name, params$weight))
-  if ( pp$parallel ) {
-    # windows
-    if ( pp$have_win ) {
-      cl <- makePSOCKcluster(pp$nr_cores)
-      registerDoParallel(cl,cores=pp$nr_cores)
-      valuesCat <- foreach(x=levels(dataS[[strata]]), .options.snow=list(preschedule=TRUE)) %dopar% {
-        genVals(
-          dataSample=dataS[dataS[[strata]] == x,],
-          dataPop=dataP[indStrata[[x]], predNames, with=FALSE],
-          params,response=dataS[dataS[[strata]] == x,eval(parse(text=params$name))],
-          typ=typ)
+  by <- params$by
+  
+  if(is.null(by) |  by != "none"){
+    if ( pp$parallel ) {
+      # windows
+      if ( pp$have_win ) {
+        cl <- makePSOCKcluster(pp$nr_cores)
+        registerDoParallel(cl,cores=pp$nr_cores)
+        valuesCat <- foreach(x=levels(dataS[[strata]]), .options.snow=list(preschedule=TRUE)) %dopar% {
+          genVals(
+            dataSample=dataS[dataS[[strata]] == x,],
+            dataPop=dataP[indStrata[[x]], predNames, with=FALSE],
+            params,response=dataS[dataS[[strata]] == x,eval(parse(text=params$name))],
+            typ=typ)
+        }
+        stopCluster(cl)
       }
-      stopCluster(cl)
-    }
-    # linux/mac
-    if ( !pp$have_win ) {
-      valuesCat <- mclapply(levels(dataS[[strata]]), function(x) {
+      # linux/mac
+      if ( !pp$have_win ) {
+        valuesCat <- mclapply(levels(dataS[[strata]]), function(x) {
+          genVals(
+            dataSample=dataS[dataS[[strata]] == x,],
+            dataPop=dataP[indStrata[[x]], predNames, with=FALSE],
+  		  params,response=dataS[dataS[[strata]] == x,eval(parse(text=params$name))],
+            typ=typ)
+        },mc.cores=pp$nr_cores)
+      }
+    } else {
+      if(params$verbose){
+        cat("All values of the by group:","\n")
+        print(levels(dataS[[strata]]))
+      }
+      valuesCat <- lapply(levels(dataS[[strata]]), function(x) {
+         if(params$verbose){
+           cat("Current by group for the binary model:",x,"\n")
+         }
         genVals(
-          dataSample=dataS[dataS[[strata]] == x,],
+          dataSample=dataS[dataS[[strata]] == x,c(predNames, additional), with=FALSE],
           dataPop=dataP[indStrata[[x]], predNames, with=FALSE],
-		  params,response=dataS[dataS[[strata]] == x,eval(parse(text=params$name))],
+  		params,response=dataS[dataS[[strata]] == x,eval(parse(text=params$name))],
           typ=typ)
-      },mc.cores=pp$nr_cores)
+      })
     }
-  } else {
-    if(params$verbose){
-      cat("All values of the by group:","\n")
-      print(levels(dataS[[strata]]))
+    
+    res <- sapply(valuesCat, class)
+    if ( any(res=="try-error") ) {
+      stop(paste0("Error in estimating the linear model. Try to specify a more simple model!\n"))
+    }
+    if ( typ=="multinom" ) {
+      response <- dataS[[params$name]]
+      valuesCat <- factor(unsplit(valuesCat, dataP[[strata]]), levels=levels(response))
+    }
+    if ( typ=="binary" ) {
+      valuesCat <- unsplit(valuesCat, dataP[[strata]], drop=FALSE)
+    }
+    if ( typ%in%c("poisson","lm","xgboost") ) {
+      valuesCat <- unsplit(valuesCat, dataP[[strata]], drop=FALSE)
+    }
+    
+  }else{
+    
+    valuesCat <- genVals(dataSample=dataS,
+                         dataPop=dataP,
+                         params,
+                         response=dataS[, eval(parse(text=params$name))],
+                         typ=typ)
+    
+    res <- sapply(valuesCat, class)
+    if ( any(res=="try-error") ) {
+      stop(paste0("Error in estimating the linear model. Try to specify a more simple model!\n"))
+    }
+    if (!typ %in% "xgboost"){
+      stop(paste0("Strata with type ", strata, "only implemented for typ xgboost, not",
+                  typ))
     }
-    valuesCat <- lapply(levels(dataS[[strata]]), function(x) {
-       if(params$verbose){
-         cat("Current by group for the binary model:",x,"\n")
-       }
-      genVals(
-        dataSample=dataS[dataS[[strata]] == x,c(predNames, additional), with=FALSE],
-        dataPop=dataP[indStrata[[x]], predNames, with=FALSE],
-		params,response=dataS[dataS[[strata]] == x,eval(parse(text=params$name))],
-        typ=typ)
-    })
   }
 
   # check for errors
-  res <- sapply(valuesCat, class)
-  if ( any(res=="try-error") ) {
-    stop(paste0("Error in estimating the linear model. Try to specify a more simple model!\n"))
-  }
-  if ( typ=="multinom" ) {
-    response <- dataS[[params$name]]
-    valuesCat <- factor(unsplit(valuesCat, dataP[[strata]]), levels=levels(response))
-  }
-  if ( typ=="binary" ) {
-    valuesCat <- unsplit(valuesCat, dataP[[strata]], drop=FALSE)
-  }
-  if ( typ%in%c("poisson","lm") ) {
-    valuesCat <- unsplit(valuesCat, dataP[[strata]], drop=FALSE)
-  }
+
+  
   return(valuesCat)
 }
 
@@ -526,7 +613,7 @@ runModel <- function(dataS, dataP, params, typ) {
 #' simulating the continuous variable. Accepted values are \code{"multinom"},
 #' for using multinomial log-linear models combined with random draws from the
 #' resulting categories, \code{"lm"}, for using (two-step) regression
-#' models combined with random error terms and \code{"poisson"} for using Poisson regression for count variables.
+#' models combined with random error terms, \code{"poisson"} for using Poisson regression for count variables, and \code{"xgboost"} for using XGBoost.
 #' @param zeros a logical indicating whether the variable specified by
 #' \code{additional} is semi-continuous, i.e., contains a considerable amount
 #' of zeros. If \code{TRUE} and \code{method} is \code{"multinom"}, a separate
@@ -636,6 +723,7 @@ runModel <- function(dataS, dataP, params, typ) {
 #' number generator to be restored.
 #' @param verbose (logical) if \code{TRUE}, additional output is written to the promt
 #' @param by defining which variable to use as split up variable of the estimation. Defaults to the strata variable.
+#' @param model_params adding optional parameter to the model, at the moment only implemented for xgboost hyperparameters
 #' @return An object of class \code{\linkS4class{simPopObj}} containing survey
 #' data as well as the simulated population data including the continuous
 #' variable specified by \code{additional} and possibly simulated categories
@@ -643,7 +731,7 @@ runModel <- function(dataS, dataP, params, typ) {
 #' @note The basic household structure and any other categorical predictors
 #' need to be simulated beforehand with the functions
 #' \code{\link{simStructure}} and \code{\link{simCategorical}}, respectively.
-#' @author Bernhard Meindl, Andreas Alfons, Alexander Kowarik (based on code by Stefan Kraft)
+#' @author Bernhard Meindl, Andreas Alfons, Alexander Kowarik (based on code by Stefan Kraft), Siro Fritzmann
 #' @references 
 #' B. Meindl, M. Templ, A. Kowarik, O. Dupriez (2017) Simulation of Synthetic Populations for Survey Data Considering Auxiliary
 #' Information. \emph{Journal of Statistical Survey}, \strong{79} (10), 1--38. \doi{10.18637/jss.v079.i10}
@@ -676,12 +764,12 @@ runModel <- function(dataS, dataP, params, typ) {
 #' # two-step regression
 #' eusilcT <- simContinuous(simPop, additional="netIncome",
 #'               regModel = "basic",
-#'               method = "lm")
+#'               method = "lm", nr_cpus=1)
 #' class(eusilcT)
 #' }
 #'
 simContinuous <- function(simPopObj, additional = "netIncome",
-  method = c("multinom", "lm","poisson"), zeros = TRUE,
+  method = c("multinom", "lm","poisson", "xgboost"), zeros = TRUE,
   breaks = NULL, lower = NULL, upper = NULL,
   equidist = TRUE, probs = NULL, gpd = TRUE,
   threshold = NULL, est = "moments", limit = NULL,
@@ -690,9 +778,9 @@ simContinuous <- function(simPopObj, additional = "netIncome",
   maxit = 500, MaxNWts = 1500,
   tol = .Machine$double.eps^0.5,
   nr_cpus=NULL, eps = NULL, regModel="basic", byHousehold=NULL,
-  imputeMissings=FALSE, seed, verbose=FALSE,by="strata") {
+  imputeMissings=FALSE, seed, verbose=FALSE,by="strata", model_params=NULL) {
 
-  x <- hhid <- vals <- id <- V1 <- randId <- NULL
+  x <- hhid <- vals <- id <- V1 <- randId <- optional_params <- NULL
 
   if ( !is.null(byHousehold) ) {
     if ( !byHousehold %in% c("mean","sum","random") ) {
@@ -703,7 +791,10 @@ simContinuous <- function(simPopObj, additional = "netIncome",
   samp <- simPopObj@sample
   pop <- simPopObj@pop
   basic <- simPopObj@basicHHvars
-  if(by=="strata"){
+  if(by %in% c("strata", "none")){
+    if(by == "none" & method[1] != "xgboost"){
+      stop(paste0("by \"none\" not implemented for method ", method[1]))
+    }
     strata <- samp@strata
   }else if(!is.null(by)){
     strata <- by
@@ -742,13 +833,13 @@ simContinuous <- function(simPopObj, additional = "netIncome",
   method <- match.arg(method)
   zeros <- isTRUE(zeros)
   log <- isTRUE(log)
-  if(log&&method=="poisson"){
+  if(log&&method %in% c("poisson", "xgboost")){
     log <- FALSE
-    warning("For Poisson regression the log=TRUE parameter is ignored and the numeric variable is not transformed.")
+    warning(paste0("For ", method," regression the log=TRUE parameter is ignored and the numeric variable is not transformed."))
   }
-  if(!is.null(alpha)&&method=="poisson"){
+  if(!is.null(alpha) && method %in% c("poisson", "xgboost")){
     alpha <- NULL
-    warning("For Poisson regression the alpha!=NULL is not yet implemented and therefore set to NULL.")
+    warning(paste0("For ", method ," regression the alpha!=NULL is not yet implemented and therefore set to NULL."))
   }
   if ( is.numeric(alpha) && length(alpha) > 0 ) {
     alpha <- rep(alpha, length.out=2)
@@ -813,13 +904,14 @@ simContinuous <- function(simPopObj, additional = "netIncome",
   # sample data of variable to be simulated
   additionalS <- dataS[[additional]]
 
-  ## determine which models to fit and do further initializations
+  ## determine which models to fit and do further initialization
   haveBreaks <- !is.null(breaks)
   if ( method == "multinom" ) {
     useMultinom <- TRUE
     useLogit <- FALSE
     useLm <- FALSE
     usePoisson <- FALSE
+    useXgboost <- FALSE
     # define break points (if missing)
     if ( haveBreaks ) {
       checkBreaks(breaks)
@@ -839,11 +931,19 @@ simContinuous <- function(simPopObj, additional = "netIncome",
     if(method=="lm"){
       useLm <- TRUE
       usePoisson <- FALSE
+      useXgboost <- FALSE
     }else if(method=="poisson"){
       useLm <- FALSE
       usePoisson <- TRUE
+      useXgboost <- FALSE
+    }else if(method=="xgboost"){
+      useMultinom <- FALSE
+      useLm <- FALSE
+      usePoisson <- FALSE
+      useXgboost <- TRUE
     }
 
+    
     if ( log ) {
       if ( is.null(const) ) {
         ## use log-transformation
@@ -879,7 +979,10 @@ simContinuous <- function(simPopObj, additional = "netIncome",
         # set control parameters
         useLogit <- zeros || any(additionalS == 0)
         useMultinom <- FALSE
-      }
+      } 
+    } else if (method == "xgboost") {
+      useLogit <- FALSE
+      useMultinom <- FALSE
     } else {
       # logistic model is used in case of semi-continuous variable
       useLogit <- zeros
@@ -949,7 +1052,8 @@ simContinuous <- function(simPopObj, additional = "netIncome",
     params$indStrata <- indStrata
     params$predNames <- predNames
     params$additional <- c(additional, weight)
-    params$verbose <- verbose
+    params$verbose <- verbose 
+    params$by <- by
     if(verbose) cat("running multinom with the following model:\n")
     if(verbose) cat(gsub("))",")",gsub("suppressWarnings[(]","",params$command)),"\n")
 
@@ -1070,12 +1174,13 @@ simContinuous <- function(simPopObj, additional = "netIncome",
     params$par <- par
     params$command <- estimationModel
     params$verbose <- verbose
+    params$by <- by
     # run in parallel if possible
 
     valuesCat <- runModel(dataS, dataP, params, typ="binary")
   }
 
-  if ( useLm || usePoisson ) {
+  if ( useLm || usePoisson || useXgboost) {
     ## some preparations
     if ( useMultinom ) {
       catLm <- names(tcat)[ncat]  # category for positive values
@@ -1132,6 +1237,9 @@ simContinuous <- function(simPopObj, additional = "netIncome",
     # auxiliary model for all strata (used in case of empty combinations)
     weights <- dataSample[[weight]]
     if(useLm){
+      # TODO{Sironimo}: fix Fehler in `contrasts<-`(`*tmp*`, value = contr.funs[1 + isOF[nn]]) : 
+      # contrasts can be applied only to factors with 2 or more levels
+      # Question{Sironimo}: use of mod and coef? -> Not used at the moment, in my opinion
       mod <- lm(formula, weights=weights, data=dataSample,x=FALSE,y=FALSE,model=FALSE)
       coef <- coef(mod)
     }else if(usePoisson){
@@ -1147,7 +1255,75 @@ simContinuous <- function(simPopObj, additional = "netIncome",
       params$command <- paste("lm(", fstring,", weights=", weight, ", data=dataSample,x=FALSE,y=FALSE,model=FALSE)", sep="")
     }else if(usePoisson){
       params$command <- paste("glm(", fstring,", weights=", weight, ", data=dataSample,family=poisson(),model=FALSE,x=FALSE,y=FALSE)", sep="")
+    }else if(useXgboost){
+      
+      # simulation via xgboost
+      if(verbose) cat("we are running xgboost:\n")
+      
+      # set xgb verbose level
+      if(verbose){
+        xgb_verbose <- 1
+      }else{
+        xgb_verbose <- 0
+      }
+      
+      if(TRUE){
+        xgb_weight <- paste0(", info = list(\"weight\" = as.numeric(dataSample$", weight, "))")
+      }else{
+        xgb_weight <- ""
+      }
+      
+      if( log ){
+        log_transform <- "log"
+      }else{
+        log_transform <- ""
+      }
+
+      pred_names <- paste(predNames[predNames != strata], collapse = "\",\"")
+      train <- paste0("xgb.DMatrix(data = model.matrix(~.+0,data = model.frame(setDT(dataSample)[,c(\"",pred_names,"\"), with=F],  na.action=na.pass)),
+                                         missing = NA, label = ", log_transform, "(dataSample$",additional,")
+                                        ", xgb_weight,")")
+      
+      # Default values
+      nrounds <- 100
+      early_stopping_rounds <- 10
+      xgb_hyper_params <- "list(nthread = 6,
+                                eta = 0.1,
+                                max_depth = 32,
+                                min_child_weight = 0,
+                                gamma = 0,
+                                subsample = 1,
+                                lambda = 0,
+                                objective = \"reg:squarederror\",
+                                eval_metric = \"rmse\")"
+      
+      if(!is.null(model_params)){
+        
+        xgb_hyper_params <- "params$model_params"
+        
+        if(!is.null(optional_params$nrounds)){
+          nrounds <- optional_params$nrounds
+        }
+        
+        if(!is.null(optional_params$early_stopping_rounds)){
+          early_stopping_rounds <- optional_params$early_stopping_rounds
+        }
+      }
+      
+      xgb_params <- paste0("nrounds = ", nrounds,",
+                            watchlist = list(train = ", train, ",
+                                             test = ", train, "), 
+                            early_stopping_rounds = ", early_stopping_rounds,",
+                            print_every_n = 10,")
+      
+      command <- paste0("xgb.train(",train, ", ",
+                                    xgb_params,
+                                    "verbose = ", xgb_verbose, ", ",
+                                    "params = ", xgb_hyper_params, ")")
+      
+      params$command <- command
     }
+    
     #params$name <- fname
     params$name <- additional
     params$excludeLevels <- excludeLevels
@@ -1164,10 +1340,14 @@ simContinuous <- function(simPopObj, additional = "netIncome",
     params$indStrata <- indStrata
     params$predNames <- predNames
     params$verbose <- verbose
+    params$by <- by
+    params$model_params <- model_params
     if(useLm){
       valuesTmp <- runModel(dataSample, dataPop, params, typ="lm")
     }else if(usePoisson){
       valuesTmp <- runModel(dataSample, dataPop, params, typ="poisson")
+    }else if(useXgboost){
+      valuesTmp <- runModel(dataSample, dataPop, params, typ="xgboost")
     }
 
     ## put simulated values together
